import * as Generated from '#design-tokens/types';
import { NuxtDesignTokens, DesignToken, NuxtThemeTokens } from '#design-tokens/types';
import { Defu } from 'defu';
import { Core } from 'browser-style-dictionary/types/browser';
import { Dictionary } from 'browser-style-dictionary/types/Dictionary';
import * as CSSType from 'csstype';

declare const palette: (color: string, suffixes?: Array<string | number>, padding?: number) => NuxtDesignTokens;

declare type DeepPartial<T> = T extends object ? {
    [P in keyof T]?: DeepPartial<T[P]>;
} : T;
declare type WrapKey<TKey, TPrefix extends string = '', TSuffix extends string = ''> = TKey extends string ? `${TPrefix}${TKey}${TSuffix}` : never;
declare type UnwrapKey<TWrappedKey, TPrefix extends string = '', TSuffix extends string = ''> = TWrappedKey extends WrapKey<infer TKey, TPrefix, TSuffix> ? TKey : '';
declare type WrappedValue<TObject extends object, TWrappedKey extends string, TPrefix extends string = '', TSuffix extends string = ''> = TObject extends {
    [K in UnwrapKey<TWrappedKey, TPrefix, TSuffix>]: infer TValue;
} ? TValue : never;
declare type PrefixObjectKeys<TObject extends object, TPrefix extends string> = {
    [K in WrapKey<keyof TObject, TPrefix>]: WrappedValue<TObject, K, TPrefix>;
};
declare type NestedKeyOf<TObject extends object> = {
    [Key in keyof TObject & (string | number)]: TObject[Key] extends object ? `${Key}` | `${Key}.${NestedKeyOf<TObject[Key]>}` : `${Key}`;
}[keyof TObject & (string | number)];
declare type WrapUnion<TObject extends string, TPrefix extends string, TSuffix extends string> = keyof {
    [K in WrapKey<TObject, TPrefix, TSuffix>]: any;
};

interface NuxtLayer {
    config: any;
    configFile: string;
    cwd: string;
}

interface ScaleTokens extends NuxtDesignTokens {
    [key: string]: {
        default?: DesignToken | NuxtDesignTokens;
        50?: DesignToken | NuxtDesignTokens;
        100?: DesignToken | NuxtDesignTokens;
        200?: DesignToken | NuxtDesignTokens;
        300?: DesignToken | NuxtDesignTokens;
        400?: DesignToken | NuxtDesignTokens;
        500?: DesignToken | NuxtDesignTokens;
        600?: DesignToken | NuxtDesignTokens;
        700?: DesignToken | NuxtDesignTokens;
        800?: DesignToken | NuxtDesignTokens;
        900?: DesignToken | NuxtDesignTokens;
    } | DesignToken;
}
interface BreakpointsTokens extends NuxtDesignTokens {
    '2xs'?: DesignToken;
    xs?: DesignToken;
    sm?: DesignToken;
    md?: DesignToken;
    lg?: DesignToken;
    xl?: DesignToken;
    '2xl'?: DesignToken;
    '3xl'?: DesignToken;
    '4xl'?: DesignToken;
    '5xl'?: DesignToken;
}
interface FontWeightTokens extends NuxtDesignTokens {
    thin?: DesignToken;
    extraLight?: DesignToken;
    light?: DesignToken;
    regular?: DesignToken;
    medium?: DesignToken;
    semiBold?: DesignToken;
    bold?: DesignToken;
    extraBold?: DesignToken;
    black?: DesignToken;
    heavyBlack: DesignToken;
}
interface GlobalTokens {
    colors?: ScaleTokens;
    fonts?: NuxtDesignTokens;
    fontWeights?: FontWeightTokens;
    fontSizes?: BreakpointsTokens;
    sizings?: BreakpointsTokens;
    spacings?: BreakpointsTokens;
    screens?: BreakpointsTokens;
    radius?: BreakpointsTokens;
    borders?: BreakpointsTokens;
    borderWidths?: BreakpointsTokens;
    shadows?: BreakpointsTokens;
    opacity: BreakpointsTokens;
    lineHeight?: BreakpointsTokens;
    lineHeights?: BreakpointsTokens;
    letterSpacings?: BreakpointsTokens;
    transitions?: NuxtDesignTokens;
    z?: NuxtDesignTokens;
}
interface NuxtStyleTheme extends NuxtDesignTokens, Defu<NuxtThemeTokens, [GlobalTokens]> {
}
declare type DefaultThemeMap = {
    gap: 'spacings';
    columnGap: 'spacings';
    rowGap: 'spacings';
    inset: 'spacings';
    insetBlock: 'spacings';
    insetBlockEnd: 'spacings';
    insetBlockStart: 'spacings';
    insetInline: 'spacings';
    insetInlineEnd: 'spacings';
    insetInlineStart: 'spacings';
    margin: 'spacings';
    marginTop: 'spacings';
    marginRight: 'spacings';
    marginBottom: 'spacings';
    marginLeft: 'spacings';
    marginBlock: 'spacings';
    marginBlockEnd: 'spacings';
    marginBlockStart: 'spacings';
    marginInline: 'spacings';
    marginInlineEnd: 'spacings';
    marginInlineStart: 'spacings';
    padding: 'spacings';
    paddingTop: 'spacings';
    paddingRight: 'spacings';
    paddingBottom: 'spacings';
    paddingLeft: 'spacings';
    paddingBlock: 'spacings';
    paddingBlockEnd: 'spacings';
    paddingBlockStart: 'spacings';
    paddingInline: 'spacings';
    paddingInlineEnd: 'spacings';
    paddingInlineStart: 'spacings';
    scrollMargin: 'spacings';
    scrollMarginTop: 'spacings';
    scrollMarginRight: 'spacings';
    scrollMarginBottom: 'spacings';
    scrollMarginLeft: 'spacings';
    scrollMarginBlock: 'spacings';
    scrollMarginBlockEnd: 'spacings';
    scrollMarginBlockStart: 'spacings';
    scrollMarginInline: 'spacings';
    scrollMarginInlineEnd: 'spacings';
    scrollMarginInlineStart: 'spacings';
    scrollPadding: 'spacings';
    scrollPaddingTop: 'spacings';
    scrollPaddingRight: 'spacings';
    scrollPaddingBottom: 'spacings';
    scrollPaddingLeft: 'spacings';
    scrollPaddingBlock: 'spacings';
    scrollPaddingBlockEnd: 'spacings';
    scrollPaddingBlockStart: 'spacings';
    scrollPaddingInline: 'spacings';
    scrollPaddingInlineEnd: 'spacings';
    scrollPaddingInlineStart: 'spacings';
    top: 'spacings';
    right: 'spacings';
    bottom: 'spacings';
    left: 'spacings';
    fontSize: 'fontSizes';
    background: 'colors';
    backgroundColor: 'colors';
    backgroundImage: 'colors';
    borderImage: 'colors';
    border: 'borders';
    borderBlock: 'borders';
    borderBlockEnd: 'borders';
    borderBlockStart: 'borders';
    borderBottom: 'borders';
    borderBottomColor: 'borders';
    borderColor: 'colors';
    borderInline: 'colors';
    borderInlineEnd: 'colors';
    borderInlineStart: 'colors';
    borderLeft: 'colors';
    borderLeftColor: 'colors';
    borderRight: 'colors';
    borderRightColor: 'colors';
    borderTop: 'colors';
    borderTopColor: 'colors';
    caretColor: 'colors';
    color: 'colors';
    columnRuleColor: 'colors';
    outline: 'colors';
    outlineColor: 'colors';
    fill: 'colors';
    stroke: 'colors';
    textDecorationColor: 'colors';
    fontFamily: 'fonts';
    fontWeight: 'fontWeights';
    lineHeight: 'lineHeights';
    letterSpacing: 'letterSpacings';
    blockSize: 'sizings';
    minBlockSize: 'sizings';
    maxBlockSize: 'sizings';
    inlineSize: 'sizings';
    minInlineSize: 'sizings';
    maxInlineSize: 'sizings';
    width: 'sizings';
    minWidth: 'sizings';
    maxWidth: 'sizings';
    height: 'sizings';
    minHeight: 'sizings';
    maxHeight: 'sizings';
    flexBasis: 'sizings';
    gridTemplateColumns: 'sizings';
    gridTemplateRows: 'sizings';
    borderWidth: 'borderWidths';
    borderTopWidth: 'borderWidths';
    borderLeftWidth: 'borderWidths';
    borderRightWidth: 'borderWidths';
    borderBottomWidth: 'borderWidths';
    borderRadius: 'radius';
    borderTopLeftRadius: 'radius';
    borderTopRightRadius: 'radius';
    borderBottomRightRadius: 'radius';
    borderBottomLeftRadius: 'radius';
    boxShadow: 'shadows';
    textShadow: 'shadows';
    transition: 'transitions';
    zIndex: 'z';
};
declare type ThemeKey<K extends keyof DefaultThemeMap> = WrapUnion<NestedKeyOf<NuxtStyleTheme[DefaultThemeMap[K]]>, `{${DefaultThemeMap[K]}.`, '}'>;
declare const defineTokens: (tokens: DeepPartial<NuxtStyleTheme>) => DeepPartial<NuxtStyleTheme>;

declare const stubTokens: (buildPath: string, force?: boolean) => Promise<void>;
declare const getStyleDictionaryInstance: (tokens: NuxtStyleTheme, buildPath: string) => Promise<Core>;
declare const generateTokens: (tokens: NuxtStyleTheme, buildPath: string, silent?: boolean) => Promise<Dictionary>;

declare const resolveConfig: (layer: NuxtLayer, key: string, configFile?: string) => {
    filePath: string;
    config: {};
};
/**
 * Resolve `tokens` config layers from `extends` layers and merge them via `defu()`.
 */
declare const resolveConfigTokens: (layers: NuxtLayer[]) => {
    tokensFilePaths: string[];
    tokens: NuxtStyleTheme;
};
declare const createTokensDir: (path: string) => Promise<void>;

declare type VuePseudos = '&:deep(' | '&:slotted(' | ':global(';
declare type AdvancedPseudos = '&::cue(' | '&::cue-region(' | '&::part(' | '&::slotted(' | '&:dir(' | '&:has(' | '&:host(' | '&:host-context(' | '&:is(' | '&:lang(' | '&:not(' | '&:nth-child(' | '&:nth-last-child(' | '&:nth-last-of-type(' | '&:nth-of-type(' | '&:where(';
declare type SimplePseudos = '&::after' | '&::backdrop' | '&::before' | '&::cue' | '&::cue-region' | '&::first-letter' | '&::first-line' | '&::grammar-error' | '&::marker' | '&::placeholder' | '&::selection' | '&::spelling-error' | '&::target-text' | '&:active' | '&:after' | '&:any-link' | '&:before' | '&:blank' | '&:checked' | '&:current' | '&:default' | '&:defined' | '&:disabled' | '&:empty' | '&:enabled' | '&:first' | '&:first-child' | '&:first-letter' | '&:first-line' | '&:first-of-type' | '&:focus' | '&:focus-visible' | '&:focus-within' | '&:fullscreen' | '&:future' | '&:hover' | '&:in-range' | '&:indeterminate' | '&:invalid' | '&:last-child' | '&:last-of-type' | '&:left' | '&:link' | '&:local-link' | '&:nth-col' | '&:nth-last-col' | '&:only-child' | '&:only-of-type' | '&:optional' | '&:out-of-range' | '&:past' | '&:paused' | '&:picture-in-picture' | '&:placeholder-shown' | '&:read-only' | '&:read-write' | '&:required' | '&:right' | '&:root' | '&:scope' | '&:target' | '&:target-within' | '&:user-invalid' | '&:user-valid' | '&:valid' | '&:visited';
declare type Pseudos = AdvancedPseudos | SimplePseudos;
interface NativeProperties extends CSSType.StandardProperties, CSSType.StandardShorthandProperties, CSSType.StandardProperties, CSSType.SvgProperties {
}
/**
 * Pseudos to be used in `css({ ':pseudo': { ...} })`
 */
declare type PseudosProperties = {
    [key in Pseudos]?: NativeProperties;
};
declare type CssProperties = NativeProperties & PseudosProperties;
declare const schemes: {
    readonly '@dark': true;
    readonly '@light': true;
};
declare type PermissiveVariants<K> = {
    [key in string]?: (K | undefined | {});
};
declare type CSS<TemplateTags extends object = {}, Theme extends object = NuxtThemeTokens> = {
    [key in keyof PrefixObjectKeys<Theme['screens'], '@screen:'>]?: (CSS<TemplateTags, Theme>);
} & {
    [key in keyof typeof schemes]?: (CSS<TemplateTags, Theme> | undefined | {});
} & {
    variants?: (PermissiveVariants<CSS<TemplateTags, Theme>>);
} & {
    [K in keyof TemplateTags]?: (CSS<TemplateTags, Theme> | undefined | {});
} & {
    [K in keyof DefaultThemeMap]?: (ThemeKey<K> | CssProperties[K] | undefined | {});
} & {
    [K in Exclude<keyof CssProperties, keyof DefaultThemeMap>]?: (CssProperties[K] | undefined | {});
} & {
    [K: string]: (number | string | CSS<TemplateTags, Theme> | {} | undefined);
};
declare const css: (declaration: CSS<{}, NuxtStyleTheme>) => CSS<{}, NuxtStyleTheme>;
declare type CssFunctionType = typeof css;

interface ModulePrivateRuntimeConfig {
    tokensDir?: string;
    server?: boolean;
    tokensFilePaths?: Array<string>;
}

declare const $tokens: (path: Generated.NuxtThemeTokensPaths, options?: Generated.TokenHelperOptions) => any;
declare type DtFunctionType = typeof $tokens;
declare const $dt: (path: Generated.NuxtThemeTokensPaths, options?: Generated.TokenHelperOptions) => any;
interface NuxtStyleConfig {
    server?: boolean;
    tokens?: Generated.NuxtThemeTokens | boolean;
}
interface ModuleOptions extends NuxtStyleConfig {
}
interface ModuleHooks {
}
interface ModulePublicRuntimeConfig {
}
declare module '@nuxt/schema' {
    interface PublicRuntimeConfig {
        style?: ModulePublicRuntimeConfig;
    }
    interface RuntimeConfig {
        style?: ModulePrivateRuntimeConfig;
    }
    interface NuxtConfig {
        style?: Partial<ModuleOptions>;
    }
    interface NuxtOptions {
        style?: ModuleOptions;
    }
}

export { $dt, $tokens, AdvancedPseudos, BreakpointsTokens, CSS, CssFunctionType, CssProperties, DefaultThemeMap, DtFunctionType, FontWeightTokens, GlobalTokens, ModuleHooks, ModuleOptions, ModulePrivateRuntimeConfig, ModulePublicRuntimeConfig, NuxtStyleConfig, NuxtStyleTheme, PermissiveVariants, Pseudos, PseudosProperties, ScaleTokens, SimplePseudos, ThemeKey, VuePseudos, createTokensDir, css, defineTokens, generateTokens, getStyleDictionaryInstance, palette, resolveConfig, resolveConfigTokens, stubTokens };
