import { defineEventHandler } from "h3";
import * as p from "paneer";
import * as r from "recast";
import toAST from "to-ast";
import { useStorage } from "#imports";
export default defineEventHandler(async () => {
  const storage = useStorage();
  const tokens = await storage.getItem("cache:design-tokens:tokens.json");
  const config = await storage.getItem("cache:design-tokens:config");
  const tokensToExpression = (_tokens) => {
    const root = r.types.builders.objectExpression([]);
    const resolveTarget = (context) => {
      if (context?.value?.arguments) {
        return context?.value?.arguments?.[0]?.properties;
      }
      if (context?.value?.properties) {
        return context.value.properties;
      }
      if (context?.properties) {
        return context.properties;
      }
      return context;
    };
    const resolveKey = (key) => {
      const charsRegex = /[`!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?~]/;
      const wordRegex = /\w+/g;
      if (key.match(wordRegex).length > 1 || key.match(charsRegex)) {
        return r.types.builders.stringLiteral(key);
      }
      return r.types.builders.identifier(key);
    };
    const walkTokens = (tokens2, context = void 0) => {
      const target = resolveTarget(context);
      Object.entries(tokens2).forEach(([key, value]) => {
        if (value.value) {
          if (value.nested) {
            const pseudosRegex = /&(::-|::|:-|:)/g;
            const nestedSelector = value.nested.replace(pseudosRegex, "");
            let nestedTarget = target.find((property) => {
              return property.key.value === value.nested || property.key.name === value.nested;
            });
            if (!nestedTarget) {
              nestedTarget = r.types.builders.objectProperty(
                resolveKey(value.nested),
                r.types.builders.objectExpression([])
              );
              target.push(nestedTarget);
            }
            delete value.nested;
            nestedTarget.value.properties.push(
              r.types.builders.objectProperty(
                resolveKey(key.replace(nestedSelector + "-", "")),
                toAST(value)
              )
            );
            return;
          }
          target.push(
            r.types.builders.objectProperty(
              resolveKey(key),
              toAST(value)
            )
          );
          return;
        }
        let newContext;
        if (value.palette) {
          delete value.palette;
          newContext = r.types.builders.objectProperty(
            resolveKey(key),
            r.types.builders.callExpression(
              r.types.builders.identifier("palette"),
              [r.types.builders.stringLiteral(value["500"].value)]
            )
          );
          target.push(newContext);
          return;
        } else {
          newContext = r.types.builders.objectProperty(
            resolveKey(key),
            r.types.builders.objectExpression([])
          );
        }
        target.push(newContext);
        walkTokens(
          value,
          newContext
        );
      });
    };
    walkTokens(_tokens, root);
    return root;
  };
  const configAst = p.parse(config);
  const defaultExport = p.defaultExport(configAst);
  defaultExport.arguments = [tokensToExpression(tokens)];
  return `
  <div style="display: flex; font-size: 10px; justify-content: space-between;">
  <pre style="line-height: 10px;">${p.generate(p.parse(config)).code}</pre>
  <pre style="line-height: 10px;">${p.generate(defaultExport).code}</pre>
  </div>
  `;
});
